# Generated by Django 6.0.3 on 2026-03-09 20:53

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='ExceptionLog',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('exception_class', models.CharField(db_index=True, max_length=255)),
                ('message', models.TextField()),
                ('traceback', models.TextField()),
                ('level', models.CharField(choices=[('debug', 'Debug'), ('info', 'Info'), ('warning', 'Warning'), ('error', 'Error'), ('critical', 'Critical')], default='error', max_length=10)),
                ('severity', models.CharField(choices=[('low', 'Low'), ('moderate', 'Moderate'), ('high', 'High'), ('critical', 'Critical')], default='high', max_length=10)),
                ('status', models.CharField(choices=[('new', 'New'), ('seen', 'Seen'), ('resolved', 'Resolved'), ('ignored', 'Ignored')], db_index=True, default='new', max_length=10)),
                ('path', models.CharField(blank=True, max_length=500)),
                ('method', models.CharField(blank=True, max_length=10)),
                ('query_params', models.JSONField(blank=True, default=dict)),
                ('body', models.TextField(blank=True)),
                ('user_info', models.CharField(blank=True, max_length=255)),
                ('ip_address', models.GenericIPAddressField(blank=True, null=True)),
                ('headers', models.JSONField(blank=True, default=dict)),
                ('view_name', models.CharField(blank=True, max_length=255)),
                ('hostname', models.CharField(blank=True, max_length=255)),
                ('environment', models.CharField(blank=True, max_length=50)),
                ('is_sent', models.BooleanField(default=False)),
                ('created_at', models.DateTimeField(auto_now_add=True, db_index=True)),
            ],
            options={
                'ordering': ['-created_at'],
            },
        ),
    ]
